@@ -101,7 +101,7 @@ def bi_fpn(features, output_channel, use_weighted_sum=True, use_sep_conv=True, i
     return out_features
 
 
-def det_head(features, filters, depth, classes=80, anchors=9, bias_init="zeros", use_sep_conv=True, activation="swish", head_activation="sigmoid", name=""):
+def det_header_pre(features, filters, depth, use_sep_conv=True, activation="swish", name=""):
     if use_sep_conv:
         names = [name + "{}_sepconv".format(id + 1) for id in range(depth)]
         convs = [keras.layers.SeparableConv2D(filters, kernel_size=3, padding="SAME", use_bias=True, name=names[id]) for id in range(depth)]
@@ -118,16 +118,18 @@ def det_head(features, filters, depth, classes=80, anchors=9, bias_init="zeros",
             nn = keras.layers.BatchNormalization(epsilon=BATCH_NORM_EPSILON, name=cur_name)(nn)
             nn = activation_by_name(nn, activation, name=cur_name + "{}_".format(id + 1))
         outputs.append(nn)
+    return outputs
 
-    if classes > 0:
-        if use_sep_conv:
-            header_conv = keras.layers.SeparableConv2D(classes * anchors, kernel_size=3, padding="SAME", bias_initializer=bias_init, name=name + "head")
-        else:
-            header_conv = keras.layers.Conv2D(classes * anchors, kernel_size=3, padding="SAME", bias_initializer=bias_init, name=name + "conv_head")
-        outputs = [header_conv(ii) for ii in outputs]
-        outputs = [keras.layers.Reshape([-1, classes])(ii) for ii in outputs]
-        outputs = tf.concat(outputs, axis=1)
-        outputs = activation_by_name(outputs, head_activation, name=name + "output_")
+
+def det_header_post(inputs, classes=80, anchors=9, bias_init="zeros", use_sep_conv=True, head_activation="sigmoid", name=""):
+    if use_sep_conv:
+        header_conv = keras.layers.SeparableConv2D(classes * anchors, kernel_size=3, padding="SAME", bias_initializer=bias_init, name=name + "head")
+    else:
+        header_conv = keras.layers.Conv2D(classes * anchors, kernel_size=3, padding="SAME", bias_initializer=bias_init, name=name + "conv_head")
+    outputs = [header_conv(ii) for ii in inputs]
+    outputs = [keras.layers.Reshape([-1, classes])(ii) for ii in outputs]
+    outputs = tf.concat(outputs, axis=1)
+    outputs = activation_by_name(outputs, head_activation, name=name + "output_")
     return outputs
 
 
@@ -139,19 +141,23 @@ def EfficientDet(
     head_depth=3,
     num_channels=64,
     use_weighted_sum=True,
+    use_object_scores=False,  # For anchor free mode
     num_anchors=9,
     num_classes=90,
     use_sep_conv=True,
     activation="swish",
     classifier_activation="sigmoid",
     freeze_backbone=False,
-    anchor_scale=4,  # Init anchors for model prediction
-    pyramid_levels=[3, 7],  # Init anchors for model prediction
     pretrained="coco",
     model_name=None,
-    rescale_mode="torch",
+    pyramid_levels=[3, 7],  # Init anchors for model prediction, not for model structure
+    anchor_aspect_ratios=[1, 2, 0.5],  # Init anchors for model predict, not for model structureion
+    anchor_num_scales=3,  # Init anchors for model prediction, not for model structure
+    anchor_scale=4,  # Init anchors for model prediction, not for model structure
+    anchor_grid_zero_start=False,  # Init anchors for model prediction, not for model structure
+    input_shape=None,  # Init anchors for model prediction, not for model structure
+    rescale_mode="torch",  # Model precessing input, not for model structure
     kwargs=None,  # Not using, recieving parameter
-    input_shape=None,  # Not using, recieving parameter
 ):
     if freeze_backbone:
         backbone.trainable = False
@@ -173,31 +179,41 @@ def EfficientDet(
         additional_feature = keras.layers.MaxPooling2D(pool_size=3, strides=2, padding="SAME", name=cur_name + "max_down")(additional_feature)
         fpn_features.append(additional_feature)
 
+    # Bi-FPN
     for id in range(fpn_depth):
         fpn_features = bi_fpn(fpn_features, num_channels, use_weighted_sum, use_sep_conv, activation=activation, name="biFPN_{}_".format(id + 1))
 
-    # Save line-width
-    head_kwargs = {"filters": num_channels, "depth": head_depth, "anchors": num_anchors, "use_sep_conv": use_sep_conv, "activation": activation}
-    bbox_out = det_head(fpn_features, classes=4, bias_init="zeros", head_activation=None, name="regressor_", **head_kwargs)
+    # Outputs
+    bboxes_features = det_header_pre(fpn_features, num_channels, head_depth, use_sep_conv, activation=activation, name="regressor_")
+    bboxes_out = det_header_post(bboxes_features, 4, num_anchors, bias_init="zeros", use_sep_conv=use_sep_conv, head_activation=None, name="regressor_")
+    if use_object_scores:
+        bias_init = tf.constant_initializer(-tf.math.log((1 - 0.01) / 0.01).numpy())
+        object_out = det_header_post(bboxes_features, 1, num_anchors, bias_init, use_sep_conv, head_activation=classifier_activation, name="object_")
+
     if num_classes > 0:
         bias_init = tf.constant_initializer(-tf.math.log((1 - 0.01) / 0.01).numpy())
-        cls_out = det_head(fpn_features, classes=num_classes, bias_init=bias_init, head_activation=classifier_activation, name="classifier_", **head_kwargs)
-        outputs = tf.concat([bbox_out, cls_out], axis=-1)
+        class_features = det_header_pre(fpn_features, num_channels, head_depth, use_sep_conv, activation=activation, name="classifier_")
+        class_out = det_header_post(class_features, num_classes, num_anchors, bias_init, use_sep_conv, classifier_activation, name="classifier_")
+        outputs = tf.concat([bboxes_out, class_out, object_out], axis=-1) if use_object_scores else tf.concat([bboxes_out, class_out], axis=-1)
     else:
-        outputs = bbox_out
+        outputs = tf.concat([bboxes_out, object_out], axis=-1) if use_object_scores else bboxes_out
     outputs = keras.layers.Activation("linear", dtype="float32", name="outputs_fp32")(outputs)
 
     model_name = model_name or backbone.name + "_det"
     model = keras.models.Model(inputs=backbone.inputs[0], outputs=outputs, name=model_name)
-    add_pre_post_process(model, rescale_mode=rescale_mode, post_process=DecodePredictions(backbone.input_shape[1:], pyramid_levels, anchor_scale))
     reload_model_weights(model, PRETRAINED_DICT, "efficientdet", pretrained)
+
+    # For prediction
+    ANCHORS = {"aspect_ratios": anchor_aspect_ratios, "num_scales": anchor_num_scales, "anchor_scale": anchor_scale, "grid_zero_start": anchor_grid_zero_start}
+    post_process = DecodePredictions(backbone.input_shape[1:], pyramid_levels, **ANCHORS, use_object_scores=use_object_scores)
+    add_pre_post_process(model, rescale_mode=rescale_mode, post_process=post_process)
     # model.backbone = backbone
     return model
 
 
 def EfficientDetD0(input_shape=(512, 512, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B0(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B0(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block1_output", "stack_4_block2_output", "stack_6_block0_output"]
     model_name = kwargs.pop("model_name", "efficientdet_d0")
     return EfficientDet(**locals(), fpn_depth=3, head_depth=3, num_channels=64, **kwargs)
@@ -205,7 +221,7 @@ def EfficientDetD0(input_shape=(512, 512, 3), freeze_backbone=False, num_classes
 
 def EfficientDetD1(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B1(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B1(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block2_output", "stack_4_block3_output", "stack_6_block1_output"]
     model_name = kwargs.pop("model_name", "efficientdet_d1")
     return EfficientDet(**locals(), fpn_depth=4, head_depth=3, num_channels=88, **kwargs)
@@ -213,7 +229,7 @@ def EfficientDetD1(input_shape=(640, 640, 3), freeze_backbone=False, num_classes
 
 def EfficientDetD2(input_shape=(768, 768, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B2(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B2(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block2_output", "stack_4_block3_output", "stack_6_block1_output"]
     model_name = kwargs.pop("model_name", "efficientdet_d2")
     return EfficientDet(**locals(), fpn_depth=5, head_depth=3, num_channels=112, **kwargs)
@@ -221,7 +237,7 @@ def EfficientDetD2(input_shape=(768, 768, 3), freeze_backbone=False, num_classes
 
 def EfficientDetD3(input_shape=(896, 896, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B3(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B3(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block2_output", "stack_4_block4_output", "stack_6_block1_output"]
     model_name = kwargs.pop("model_name", "efficientdet_d3")
     return EfficientDet(**locals(), fpn_depth=6, head_depth=4, num_channels=160, **kwargs)
@@ -229,7 +245,7 @@ def EfficientDetD3(input_shape=(896, 896, 3), freeze_backbone=False, num_classes
 
 def EfficientDetD4(input_shape=(1024, 1024, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B4(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B4(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block3_output", "stack_4_block5_output", "stack_6_block1_output"]
     model_name = kwargs.pop("model_name", "efficientdet_d4")
     return EfficientDet(**locals(), fpn_depth=7, head_depth=4, num_channels=224, **kwargs)
@@ -237,7 +253,7 @@ def EfficientDetD4(input_shape=(1024, 1024, 3), freeze_backbone=False, num_class
 
 def EfficientDetD5(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B5(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B5(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block4_output", "stack_4_block6_output", "stack_6_block2_output"]
     model_name = kwargs.pop("model_name", "efficientdet_d5")
     return EfficientDet(**locals(), fpn_depth=7, head_depth=4, num_channels=288, **kwargs)
@@ -245,7 +261,7 @@ def EfficientDetD5(input_shape=(1280, 1280, 3), freeze_backbone=False, num_class
 
 def EfficientDetD6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B6(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B6(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block5_output", "stack_4_block7_output", "stack_6_block2_output"]
     model_name = kwargs.pop("model_name", "efficientdet_d6")
     return EfficientDet(**locals(), fpn_depth=8, head_depth=5, num_channels=384, use_weighted_sum=False, **kwargs)
@@ -253,7 +269,7 @@ def EfficientDetD6(input_shape=(1280, 1280, 3), freeze_backbone=False, num_class
 
 def EfficientDetD7(input_shape=(1536, 1536, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B6(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B6(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block5_output", "stack_4_block7_output", "stack_6_block2_output"]
     anchor_scale = kwargs.pop("anchor_scale", 5)
     model_name = kwargs.pop("model_name", "efficientdet_d7")
@@ -262,7 +278,7 @@ def EfficientDetD7(input_shape=(1536, 1536, 3), freeze_backbone=False, num_class
 
 def EfficientDetD7X(input_shape=(1536, 1536, 3), freeze_backbone=False, num_classes=90, backbone=None, activation="swish", pretrained="coco", **kwargs):
     if backbone is None:
-        backbone = efficientnet.EfficientNetV1B7(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation, pretrained=None)
+        backbone = efficientnet.EfficientNetV1B7(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block6_output", "stack_4_block9_output", "stack_6_block3_output"]
     pyramid_levels = kwargs.pop("pyramid_levels", [3, 8])
     additional_features = max(pyramid_levels) - 5  # 8 -> 3, 7 -> 2
